#include <conio.h>
#include <stdio.h>
#include <mem.h>
#include <ctype.h>
#include <string.h>

#define LINECOUNT 60
#define LINELEN   20

char lines[LINECOUNT][LINELEN+1];		/* Extra Space for Line Terminate */

#define LMARGIN  6
#define SPACING  4
#define TMARGIN  2
#define CCOUNT   3
#define LCOUNT   20

int offset=4;

int screenx( int x , int y ) {
	int c;
	c= y/LCOUNT;
	return( LMARGIN + (c*(SPACING+LINELEN)) +x );
	}

int screeny( int y ) {
	int c;
	c= y/LCOUNT;
	return( TMARGIN + (y-(c*LCOUNT))  );
	}

void refresh( int x, int y) {
	gotoxy( 70, 24 );
	cprintf("X:%02d Y:%02d",x,y);
	}

void refreshline(int y) {
	if (y) {
		gotoxy( screenx(0,y) , screeny(y) );
		cprintf("%*.*s",LINELEN,LINELEN,lines[y]);
		}
	else {
		gotoxy( screenx(offset,y) , screeny(y) );
		cprintf("%*.*s",LINELEN-offset,LINELEN-offset,lines[y]+offset);
		}
	}

void refreshscreen(void) {
	int y;
	for(y=0;y<LINECOUNT;y++) refreshline(y);
	}

void centerline( int y ) {
	char buffer[LINELEN];
	int len,baselen;
	char *s,*t;
	baselen= y?LINELEN:LINELEN-offset;
	len=baselen;
	s= lines[y]+(y?0:offset);
	while (len && *s==' ') {
		s++;
		len--;
		}
	t=&lines[y][LINELEN-1];
	while (len && *t==' ') {
		t--;
		len--;
		}
	strncpy(buffer,s,len);
	memset(lines[y],' ',LINELEN);
	strncpy( lines[y]+((baselen-len)/2)+(y?0:offset) , buffer , len );
	}

#define UPARROW 	256+72
#define DOWNARROW 	256+80
#define LEFTARROW 	256+75
#define RIGHTARROW 	256+77
#define PAGEUP		256+73
#define PAGEDOWN 	256+81
#define HOME		256+71
#define CTRLHOME	256+119
#define END		256+79
#define CTRLEND		256+117
#define INSERT		256+82
#define CTRLINSERT	256+146
#define ALTINSERT	256+162
#define DELETE		256+83
#define CTRLDELETE	256+147
#define ALTDELETE	256+163
#define CTRLENTER	10
#define CTRLBACKSPACE	127
#define ESC 		27
#define TAB		'\t'
#define BACKSPACE 	'\b'
#define RETURN  	13

int getkey(void) {
	int key;
	key=getch();
	if (!key) {
		key=getch()+256;
		}
	return(key);
	}

void edit(void) {
	int x,y,sx,sy;
	x=offset;
	y=0;
	while (1) {
		int key;
		sx=screenx(x,y);
		sy=screeny(y);
		gotoxy(sx,sy);
		putch( lines[y][x] );
		refresh(x,y);
		gotoxy(sx,sy);
		key=getkey();
		switch (key) {
			case ESC: return;
			case RETURN:
				x=0;
				if (y<LINECOUNT-1) y++;
				break;
			case BACKSPACE:
				if (x) {
					if (y || x>offset) {
						x--;
						}
					}
				else if (y) {
					y--;
					x=LINELEN-1;
					}
				lines[y][x]=' ';
				break;
			case HOME:
				if (!y) x=offset;
				else x=0;
				break;
			case CTRLHOME:
				x=offset;
				y=0;
				break;
			case END:
				x=LINELEN-1;
				while(lines[y][x]==' ') {
					x--;
					if (!y && x<=offset) break;
					if (!x) break;
					}
				if (x<LINELEN-1) x++;
				break;
			case CTRLEND:
				y=LINECOUNT-1;
				x=LINELEN-1;
				while ( lines[y][x]==' ') {
					if (y==0 && x==offset) break;
					if (x) x--;
					else {
						y--;
						x=LINELEN-1;
						}
					}
				if ( lines[y][x] != ' ' ) {
					if (x<LINELEN-1) x++;
					else if (y<LINECOUNT-1) {
						y++;
						x=0;
						}
					}
				break;
			case UPARROW:
				if (y>1 || (y==1 && x>=offset) ) y--;
				break;
			case DOWNARROW:
				if (y<LINECOUNT-1) y++;
				break;
			case LEFTARROW:
				if (!x && y) {
					y--;
					x=LINELEN-1;
					}
				else if (y || x>offset) x--;
				break;
			case RIGHTARROW:
				if (x<LINELEN-1) x++;
				else if (y<LINECOUNT-1) {
					y++;
					x=0;
					}
				break;
			case PAGEUP:
				y = ( (y-1) /4 ) *4;
				if (y==0) x=offset;
				else x=0;
				break;
			case PAGEDOWN:
				y+=4;
				if (y>=LINECOUNT) y=LINECOUNT-4;
				else y=(y/4)*4;
				x=0;
				break;
			case TAB:
				centerline(y);
				refreshline(y);
				break;
			case INSERT:
				memmove( &( lines[y][x+1] ) , &( lines[y][x] ) , LINELEN-x-1 );
				lines[y][x]=' ';
				refreshline(y);
				break;
			case CTRLENTER:
				memmove( lines[y+1] , lines[y] , (LINECOUNT-y-1)*sizeof(lines[y]) );
				memset( lines[y] , ' ' , LINELEN );
				refreshscreen();
				break;
			case DELETE:
				memmove( &( lines[y][x] ) , &( lines[y][x+1] ) , LINELEN-x-1 );
				lines[y][LINELEN-1]=' ';
				refreshline(y);
				break;
			case CTRLBACKSPACE:
				memmove( lines[y] , lines[y+1] , (LINECOUNT-y-1)*sizeof(lines[y]) );
				memset( lines[LINECOUNT-1] , ' ' , LINELEN );
				refreshscreen();
				break;
			default:
				if (isprint(key)) {
					lines[y][x]=key;
					putch(key);
					if (x<LINELEN-1) x++;
					else if (y<LINECOUNT-1) {
						x=0;
						y++;
						}
					}
				break;
			}
		}

	}

void drawscreen(char* filename) {
	int loop,loopc;
	int x,y;
	clrscr();
	gotoxy(1,24);
	cprintf("BEEPER EDIT V2.0 (c)1996 Joshua M. Levine %26.26s X:%02d Y:%02d\n\r",filename,4,1);
	cprintf("ESC QuitTAB CenterDEL/INSHOME/ENDCTRL-ENTER/BKSPCTRL-HOME/ENDPAGEUP/DN%c%c%c",0x18,0x19,0x1d);
	for(loopc=0;loopc<CCOUNT;loopc++) {
		x=LMARGIN-1+loopc*(LINELEN+SPACING);
		y=TMARGIN-1;
		gotoxy(x,y);
		putch('');
		for(loop=0;loop<LINELEN;loop++) {
			putch('');
			}
		putch('');
		y++;
		for(loop=0;loop<LCOUNT;loop++) {
			gotoxy(x,y);
			if (loop%4==0) putch('');
			else putch('');
			cprintf("%20.20s",lines[loopc*LCOUNT+loop]);
			gotoxy(x+LINELEN+1,y);
			if (loop%4==0) putch('');
			else putch('');
			y++;
			}
		gotoxy(x,y);
		putch('');
		for(loop=0;loop<LINELEN;loop++) {
			putch('');
			}
		putch('');
		}
	if (offset>0) {
		int x;
		gotoxy(LMARGIN,TMARGIN);
		for(x=0;x<offset;x++) {
			putch('');
			}
		}
	}

int yesno(void) {
	int key;
	do key=toupper(getch()); while (!strchr("YN",key));
	return(key=='Y');
	}

void main1(void) {
	int x;
	do {

		x=getkey();
		if (x) cprintf("%d\n\r",x);
		} while (x!=ESC);
	}

void main( int argc, char **argv ) {
	FILE *f;
	if (argc<2) {
		cprintf("***** BEEPER EDIT, (c)1996 Joshua Levine, P.O. BOX 382, NY NY 10268\n\r");
		cprintf("Usage: BEDIT filename [X]\n\r");
		cprintf("Where: X indicates the beeper does not reserve\n\r");
		cprintf("       the first four letters for mail box number.\n\r");
		return;
		}
	if (argc==3) {
		if (toupper(argv[2][0])=='X') offset=0;
		else {
			cprintf("ERROR: Unknown argument '%s'\n\r",argv[2]);
			return;
			}
		}
	memset(lines,' ', sizeof(lines) );
	f=fopen( argv[1] , "rb" );
	if (!f) {
		cprintf("ERROR: Could not open '%s'\n\r",argv[1]);
		cprintf("Create it (y/n)?");
		if (!yesno()) return;
		f=fopen( argv[1] , "wb" );
		if (!f) {
			cprintf("ERROR: Could not create '%s'\n\r",argv[1]);
			return;
			}
		}
	else {
		fread( ((char *) lines)+offset , 1 , sizeof(lines)-offset , f );
		}
	fclose(f);
	strupr(argv[1]);
	drawscreen(argv[1]);
	edit();
	gotoxy(1,25);
	clreol();
	gotoxy(1,24);
	clreol();
	cprintf("Save changes to file (y/n)?");
	if (yesno()) {
		int loop;
		char *s;
		char *b=lines[0]+offset;
		for(loop=0;loop<LINECOUNT;loop++) lines[loop][LINELEN]=' ';
		s= &lines[LINECOUNT-1][LINELEN];
		while (s>b && *s==' ') s--;
		s++;
		*s=0x00;
		f=fopen(argv[1],"wb");
		if (!f) {
			gotoxy(1,24);
			clreol();
			cprintf("ERROR: Could not open file!\n\r");
			return;
			}
		fputs(b,f);
		fclose(f);
		}
	gotoxy(1,24);
	clreol();
	}




