/* TQCOM.c   By Joshua M. Levine  July 1990 */
/* COM Port Services for TigerQuote Server  */

#include <dos.h>        /* Defines Outportb(), Disable(), and Enable() */
#include <bios.h>	/* prototypes bioscom() for setting baud */
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <alloc.h>
#include <conio.h>

#include "com.h"

#define STX (2)
#define CR (13)

#define IER 1     /* interrupt enable port */
#define IIR 2     /* interrupt identification port */
#define LCR 3     /* line control register */
#define MCR 4     /* modem control register */
#define LSR 5     /* line status register */

#define RECINT 4	/* Mask on IIR for Char Received */

#define IMR 0x21  			/* inturrupt mask register */
#define MASK(x) (~( (char) (1 << (x))))	/* inturrupt mask */
#define VECTOR(x) (x)+8			/* service intterrupt vector */

#define SEND(outbyte) outportb( combase, outbyte)	/* transmit via card*/

#define INTDONE outportb( 0x20, 0x20)  /* signal inturrupt serviced */

#define CLEAR2SEND (inportb(combase+LSR) & 32)     /* Holding reg clear? */

	/* OutPut */

/*  int putcom(char outbyte);      		*/
/*   	Sends OUTBYTE through port. 		*/

void process(char *message);

int ports[]= {
	0x3f8,
	0x2f8,
	0x3e8,
	0x2e8
	};

int combase;

void putcom( unsigned char outbyte)  {
	while (!CLEAR2SEND);
	SEND(outbyte);
	return;
	}

int ready2send(void) {			/* Returns 1 if send buffer clear */
	return(CLEAR2SEND);
	}

	/* InPut */

#define BUFFLEN 10000

char buffer[2][BUFFLEN+1];	/* Doubble buffer incoming data */

int inbuff=0;			/* Wich buffer is doing what */
int prbuff=0;

char *next=NULL;		/* Next location */
char *end=NULL;       /* End of buffer */

int overflow=0;			/* Incoming message was missed */
int linestat=0;                 /* Current linstatus */

int dumpmode=0;

int semaphore=0;		/* Don't change game table now */

int processnow=0;

void triggerprocess(void) {
	static inside=0;
	disable();
	if (inside) {
		error("INSIDE!!!!!!");
		enable();
		}
	else if (processnow && !semaphore) {
		inside=1;
		processnow=0;
		enable();
		process(buffer[prbuff]);
		inside=0;
		prbuff=inbuff;
		}
	else {
		enable();
		}
	}

void hold(void) {
	if (semaphore) error("Semaphore ReLocked!");
	semaphore=1;
	}

void release(void) {
	if (!semaphore) error("Nonexistant Semaphore unlocked!");
	else {
		semaphore=0;
		triggerprocess();
		}
	}

#define BSCREEN (char far *)0xb8000000UL
#define CSCREEN (char far *)0xb0000000UL

char far *bscreen=BSCREEN;
char far *cscreen=CSCREEN;

void putscr(char x) {
	*bscreen=x;
	*cscreen=x;
	bscreen+=2;
	if (bscreen>= (BSCREEN+ 80*24*2) ) bscreen=BSCREEN;
	cscreen+=2;
	if (cscreen>= (CSCREEN+ 80*24*2) ) cscreen=CSCREEN;
	}

void interrupt rec(void) {
	while ( inportb(combase+IIR) & RECINT ) {
		unsigned char x;
		linestat |= inportb( combase+LSR );
		x = inportb( combase );
		if (x == STX) {
			if (dumpmode) putscr('');
			next=buffer[inbuff];
			end=buffer[inbuff]+BUFFLEN;
			}
		else if (next) {
			if ( x == 0x04 ) {
				*next=0x00;
				next=NULL;
				if (dumpmode) {
					putscr('');
					}
				if (prbuff!=inbuff) {
					next=NULL;
					overflow=1;
					error("Comport overflow");
					}
				else {
					inbuff = inbuff?0:1;
					processnow=1;
					}
				}

			else if (next<end) {
				*next=x;
				next++;
				if (dumpmode) {
					putscr(x);
					}
				}
			}
		}
	INTDONE;
	enable();
	if (processnow && !semaphore) triggerprocess();
	}

void interrupt (*comoldvect)();				/* storage for old vector handler */

int saveirq;

void killcom() {					/* Restore machine State */
	outportb( (int) combase+IER, (char) 0 );	/* Turn off com related INTs */
	outportb( (int) combase+MCR, (char) 0 );	/* turn off out-2 before exiting */
	setvect( VECTOR(saveirq), comoldvect);		/* Restore old interrupt handle, if available */
	}

int initcom(unsigned port, int irq) {			/* inialize TID output */
	int x;
	if (port<1 || port>4) {
		error("Invalid Port!");
		return(1);
		}

	combase=ports[port-1];
	linestat=0;
	saveirq=irq;
	outportb(combase+IER, 0);			/* Stop interrupts */
	outportb( (int) combase+MCR, 8+4+3); 		/* enable OUT2 for interrupts */

	comoldvect= getvect(VECTOR(irq));		/* Save old vector for later */
	setvect( VECTOR(irq), rec);                     /* set sevice driver */

	disable();
	x=inportb(IMR);					/* Get current INTS */
	x  = ~x ;
	x |= ( 1 << irq );
	x =  ~x ;
	outportb( IMR , x );
	inportb( combase );
	outportb( (int) combase+IER, 1 ); 		/* Interrupt on recive */
	inportb( combase + LSR);
	enable();
	return(0);
	}

int checkcom(void) {				/* !0=Error */
	int ret=0;
	if (linestat&2) {
	   linestat^=2;
	   error("ComPort Overrun");
	   ret=1;
	   }
	if (linestat&4) {
	   linestat^=4;
	   error("Parity Error");
	   ret=1;
	   }
	if (linestat&8) {
	   linestat^=8;
	   error("Framing Error");
	   ret=1;
	   }
	if (linestat&16) {
	   linestat^=16;
	   error("Break Condition");
	   ret=1;
	   }
	if (overflow) {
	     error("Comport Overflow");
	     ret=1;
	     }
	return(ret);
	}
