/* TQCOM.c   By Joshua M. Levine  July 1990 */
/* COM Port Services for TigerQuote Server  */

#include <dos.h>        /* Defines Outportb(), Disable(), and Enable() */
#include <bios.h>	/* prototypes bioscom() for setting baud */
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <alloc.h>
#include <conio.h>
#include <stdio.h>

/*  #include "com.h" */

#define STX (2)
#define CR (13)

#define IER 1     /* interrupt enable port */
#define IIR 2     /* interrupt identification port */
#define LCR 3     /* line control register */
#define MCR 4     /* modem control register */
#define LSR 5     /* line status register */

#define IMR 0x21  			/* inturrupt mask register */
#define MASK(x) (~( (char) (1 << (x))))	/* inturrupt mask */
#define VECTOR(x) (x)+8			/* service intterrupt vector */

#define SEND(outbyte) outportb( combase, outbyte)	/* transmit via card*/

#define INTDONE outportb( 0x20, 0x20)  /* signal inturrupt serviced */

#define CLEAR2SEND (inportb(combase+LSR) & 32)     /* Holding reg clear? */

	/* OutPut */

/*  int putcom(char outbyte);      		*/
/*   	Sends OUTBYTE through port. 		*/

void process(char *message);

int ports[]= {
	0x3f8,
	0x2f8,
	0x3e8,
	0x2e8
	};

int irqs[]= {
	4,
	3,
	4,
	3
	};

int combase;

#define BUFFLEN 30000

char buffer[BUFFLEN];		/* Buffer incoming data */

char * volatile new;		/* New incomming goes here */
char *next;			/* Next location to read */
char *end;       		/* End of buffer */

int overflow=0;			/* Incoming message was missed */
int linestat=0;                 /* Current linstatus */

void interrupt rec(void) {
	unsigned char x;
	char *y;
	x = inportb( combase );
	linestat |= inportb( combase+LSR );
	INTDONE;
	y=new;
	y++;
	if (y==end) y=buffer;
	if (y==next) {
		overflow=1;
		}
	else {
		*y=x;
		new=y;
		}
	}

void interrupt (*comoldvect)();				/* storage for old vector handler */

int saveirq;

void killcom(void) {					/* Restore machine State */
	outportb( (int) combase+IER, (char) 0 );	/* Turn off com related INTs */
	outportb( (int) combase+MCR, (char) 0 );	/* turn off out-2 before exiting */
	setvect( VECTOR(saveirq), comoldvect);		/* Restore old interrupt handle, if available */
	}

int initcom(unsigned port) {			/* inialize TID output */
	int x;
	int irq;
	if (port<1 || port>4) {
		cprintf("Invalid Port!");
		return(1);
		}

	combase=ports[port-1];
	irq=irqs[port-1];
	linestat=0;
	saveirq=irq;
	outportb(combase+IER, 0);			/* Stop interrupts */
	outportb( (int) combase+MCR, 8+4+3); 		/* enable OUT2 for interrupts */

	comoldvect= getvect(VECTOR(irq));		/* Save old vector for later */
	setvect( VECTOR(irq), rec);                     /* set sevice driver */

	disable();
	x=inportb(IMR);					/* Get current INTS */
	x  = ~x ;
	x |= ( 1 << irq );
	x =  ~x ;
	outportb( IMR , x );
	inportb( combase );
	outportb( (int) combase+IER, 1 ); 		/* Interrupt on recive */
	inportb( combase + LSR);
	enable();
	new=next=buffer;
	end=buffer+BUFFLEN;
	return(0);
	}

void put(char s) {
	putch(s);
	if (biosprint(2,0,0) & 0x10) biosprint(0,s,0);
	}

void putstr(char *s) {
	while (*s) {
		put(*(s++));
		}
	}

void error(char *s) {
	putstr("****** ERROR ****** ");
	putstr(s);
	}

int checkcom(void) {				/* !0=Error */
	int ret=0;
	if (linestat&2) {
	   linestat^=2;
	   error("ComPort Overrun");
	   ret=1;
	   }
	if (linestat&4) {
	   linestat^=4;
	   error("Parity Error");
	   ret=1;
	   }
	if (linestat&8) {
	   linestat^=8;
	   error("Framing Error");
	   ret=1;
	   }
	if (linestat&16) {
	   linestat^=16;
	   error("Break Condition");
	   ret=1;
	   }
	if (overflow) {
	     error("Comport Overflow");
	     ret=1;
	     }
	return(ret);

	}

char readcom(void) {
	char *y;
	checkcom();
	if (next==new) return(0);
	y=next;
	y++;
	if (y==end) y=buffer;
	next=y;
	return(*y);
	}

#define FILTERCOUNT 200

char filters[FILTERCOUNT][3];

void main( int argc, char *argv[] ) {
	int port;
	int flag=0;
	char buffer[80];
	int blen=0;
	int linelen;
	int filtercount=0;
	FILE *f;
	cprintf("*** FILTER, By Joshua Levine, (c)1992 Joshua Levine\n\r");
	if (argc==1) {
		cprintf("Usage: FILTER comport\n\r");
		cprintf("Where: comport is 1-4\n\r");
		cprintf("       MATCH.FLT is a file that hold a list of three letter match codes\n\r");
		cprintf("       to look for in BC- messages\n\r");
		return;
		}
	lowvideo();
	f=fopen("MATCH.FLT","rb");
	if (!f) {
		cprintf("Could not open MATCH.FLT\n\r");
		return;
		}
	putstr("\n\rFILTERS:\n\r");
	linelen=0;
	while (EOF!=fscanf(f," %3s" , buffer)) {
		linelen+=strlen(buffer)+1;
		if (linelen>=80) {
			putstr("\n\r");
			linelen=strlen(buffer)+1;
			}
		strncpy( filters[filtercount++] , buffer , 3);
		putstr(buffer);
		putstr(" ");
		if (filtercount==FILTERCOUNT) {
			cprintf("\n\rToo many filters!\n\r");
			break;
			}
		}
	fclose(f);
	putstr("\n\r\n\r");
	port=atoi(argv[1]);
	if (initcom(port)) {
		cprintf("Error opening COM%d.\n\r",port);
		return;
		}
	cprintf("Using COM%d.\n\r",port);
	while ( !kbhit() || getch()!=0x1b ) {
		char x;
		x=readcom();
		if (!x) continue;
		if (x==0x02) {
			flag=1;
			blen=0;
			lowvideo();
			}
		else if (flag==1) {
			if (x=='\r') {
                        	int loop;
				for(loop=0;loop<filtercount;loop++) {
					if (!strncmp(buffer+3,filters[loop],strlen(filters[loop]) )) {
						sound(2000);
						putstr(filters[loop]);
						putstr(" CODE -----------\n\r");
						highvideo();
						flag=2;
						nosound();
						break;
						}
					}
				if (flag==1) flag=0;
				}
			else if (blen<80) {
				buffer[blen++]=x;
				}
			}
		else if (flag==2) {
			put(x);
			if (x=='\r') put('\n');
			}
		else {
			putch(x);
			if (x=='\r') putch('\n');
			}
		}
	killcom();
	cprintf("\n\rGoodbye\n\r");
	}



