#include <stdio.h>
#include <conio.h>
#include <ctype.h>
#include <time.h>
#include <stdlib.h>
#include <string.h>
#include <dos.h>
#include <stdarg.h>

#include "com.h"

struct game {
	char sport[2];
	struct date day;
	int visitteam;
	int hometeam;

	int visitscore;
	int homescore;
	int status;		/* 0=Not Started , >90 special */
	int remain;		/* Time remaining 0=Na, second othwerwise */
	int updateflag;		/* Changed since last read? */

	char visitpitch[12];
	char visitmessage[6];		/* PROBL or CONFRM or (10TH) */
	char homepitch[12];
	char homemessage[6];

	char visitcpitch[12];		/* Confirmed Pitchers */
	char homecpitch[12];

	unsigned seq;
	char box;			/* Box number N=National,A=American */
	char time[5];			/* Time of game */
	char visitline[4];
	char homeline[4];
	char overunder[4];
	};

#define ERRORCOUNT 10
char errors[ERRORCOUNT][80];
int nexterror=0;

void error(char *s) {
	strncpy( errors[nexterror] , s , 80);
	nexterror++;
	if (nexterror==ERRORCOUNT) nexterror=0;
	}

#define GAMECOUNT 150

struct game *games;

struct game *gamebuffer;

unsigned gamecount=0;

int datecmp( const struct date *date1 , const struct date *date2) {
	int x;
	x= date1->da_year - date2->da_year;
	if (x) return(x);
	x= date1->da_mon - date2->da_mon;
	if (x) return(x);
	x= date1->da_day - date2->da_day;
	return(x);
	}

int gamecmp( const struct game *x , const struct game *y ) {
	int t;
	t=(x->sport[0]) - (y->sport[0]);
	if (t) return(t);
	t=(x->sport[1]) - (y->sport[1]);
	if (t) return(t);
	t=datecmp( &(x->day) , &(y->day));
	if (t) return(t);
	t= (x->visitteam) - (y->visitteam);
	return(t);
	}

void *binsert(const void *key, void *base, size_t *num, size_t width, int (*fcmp)(const void *, const void *)) {
	char *x=base;
	size_t count=*num;
	while ( count && ( (fcmp)(key,x) >= 0 ) ) {
		count--;
		x += width;
		}
	memmove( x+width , x , width * count );
	memcpy( x , key , width);
	(*num)++;
	return(x);
	}

int get3digits( char *s ) {
	unsigned x=0;
	int y=3;
	while (isspace(*s) && y) {
		y--;
		s++;
		}
	while (y--) {
		if (!isdigit(*s)) return(-1);
		x *= 10;
		x += *s - '0';
		s++;
		}
	return( x);
	}


void blankgame( struct game *h ) {	/* Blanks out all non key fields */
	h->visitscore=0;
	h->homescore=0;
	h->status=0;
	h->remain=0;
	h->updateflag=0;

	h->visitpitch[0]=0x00;
	h->homepitch[0]=0x00;
	h->visitmessage[0]=0x00;		/* Either CONFRM or PROB */
	h->homemessage[0]=0x00;

	h->visitcpitch[0]=0x00;		/* Confirmed Pitchers */
	h->homecpitch[0]=0x00;

	h->seq=0;
	h->box=' ';			/* Box number N=National,A=American */
	h->time[0]=0x00;			/* Time of game */
	h->visitline[0]=0x00;
	h->homeline[0]=0x00;
	h->overunder[0]=0x00;
	}

void strtran( char *a , int s , int d, int c ) {  /* Convert every S into D until reaching c*/
	while (*a) {
		if (*a==s) *a=d;
		if (*a==c) break;
		a++;
		}
	}

void cleanstr( char *s ) {		/* Remove any non-printables */
	while (*s) {
		if (!isprint(*s)) *s='~';
		s++;
		}
	}

struct date today;

int str2date( struct date *d , const char *s ) {
	d->da_mon = atoi(s);
	d->da_day = atoi(s+3);
	d->da_year= atoi(s+6);
	return(0);
	}

char *dow( struct date *d) {	/* Returns 2 letter Day of Week */
	static char *days[]={ "SU","MO","TU","WE","TH","FR","SA","??" };
	struct tm time_check;

	   time_check.tm_year = d->da_year - 1900;
	   time_check.tm_mon  = d->da_mon - 1;
	   time_check.tm_mday =d->da_day;
	   time_check.tm_hour = 0;
	   time_check.tm_min  = 0;
	   time_check.tm_sec  = 1;
	   time_check.tm_isdst = -1;

	/*  call mktime to fill in the weekday field of the structure */
	   if (mktime(&time_check) == -1)
	      time_check.tm_wday = 7;

	return( days[time_check.tm_wday] );
	}

typedef int (*fptr)(const void *, const void *);

void process( char *b ) {
	struct game g;
	struct game *f;
	int confirmed=0;
	int x;		/* Args read by sscanf */
	char s;		/* Partial or Final */
	char *p;
	int secs,mins;
	strtran( b , ' ' , '0', '\r' );
	x=sscanf( b,  "BC-%2s%c+%3u:%3u*%3u%3u%2uR%2u:%2u" , g.sport , &s , &g.visitteam, &g.hometeam , &g.visitscore , &g.homescore , &g.status , &mins , &secs );
	if (x==9) g.remain = mins*60 + secs;
	else if (x==7) g.remain=-1;
	else return;

	if ( strncmp(g.sport,"AA",2)  && strncmp(g.sport,"AB",2) && strncmp(g.sport,"AF",2) ) return;
	if (!strchr("EPFQR",s)) return;		/* Not interested in message type */

	g.day=today;

	f=bsearch( &g , games , gamecount , sizeof(struct game) , (fptr) gamecmp );
	if (!f) {
		if (gamecount>=GAMECOUNT) return;
		f=binsert( &g , games, &gamecount , sizeof( struct game) , (fptr) gamecmp );
		blankgame( f );
		}

	if (f->visitscore != g.visitscore) {
		f->visitscore = g.visitscore;
		f->updateflag =1;
		}
	if (f->homescore  != g.homescore) {
		f->homescore = g.homescore;
		f->updateflag=1;
		}
	f->remain = g.remain;
	f->status = g.status;

/*     PITCHING: CAL -  S.LEWIS-(7TH)  VS. MIN -  M.GUTHRIE-(7TH)  */

	if (strstr( b , "PITCHERS CONFIRMED" )) confirmed=1;

	p=strstr( b , "PITCHING:");
	if (p) {
		p=strtok( p , " -");		/* Skip PITCHING: */
		p=strtok( NULL , " -");		/* Skip CAL */
		p=strtok( NULL , " -");		/* Got S.LEWIS */
		if (p) {
			if ( isalpha(*p) && *(p+1)=='.') {	/* Skip S. */
				strncpy( f->visitpitch , p+2 , 12);
				}
			else {
				strncpy( f->visitpitch , p , 12 );
				}
			}
		p=strtok( NULL , " -");
		if (p && *p=='(') {
			strncpy( f->visitmessage , p , 6);
			p=strtok( NULL , "- ");
			}

		if ( strcmp(p,"VS.") ) error("VS. Not found");

		strtok( NULL , " -");		/* Skip MIN */

		p=strtok( NULL , " -");		/* Got M.GUTHRIE */
		if (p) {
			if ( isalpha(*p) && *(p+1)=='.') {	/* Skip S. */
				strncpy( f->homepitch , p+2 , 12);
				}
			else {
				strncpy( f->homepitch , p , 12 );
				}
			}
		p=strtok( NULL , " -\n");
		if (p && *p=='(') {
			strncpy( f->homemessage , p , 6);
			p=strtok( NULL , "- ");
			}
		}

	if (confirmed) {
		strncpy( f->visitmessage, "CONFRM" , 6);
		strncpy( f->visitcpitch , f->visitpitch , 12);

		strncpy( f->homemessage, "CONFRM" , 6);
		strncpy( f->homecpitch , f->homepitch , 12);
		f->updateflag=1;
		}
	}

#define TEAMLEN	20

char teams[1000][TEAMLEN+1];	/* There are only 3 digits for team, so 1000 will do it */

void readteams( char *filename ) {
	FILE *f;
	static char buffer[80];
	f=fopen(filename,"r");
	if (!f) {
		fprintf( stderr, "Could not open %s for team info.\n\r", filename);
		exit(1);
		}
	while (fgets( buffer , 80 , f )) {
		char *s;
		unsigned team;
		s=strtok( buffer , " ");
		team = s ? atoi(s) : 0;
		if (team) {
			s=strtok( NULL, "\n" );
			strncpy( teams[team] , s , TEAMLEN );
			}
		}
	fclose(f);
	}

void showgame( int x) {
	struct game *g;
	if (x<1 || x>gamecount) {
		cprintf("\n\r\n\rNO MORE GAMES\n\r");
		return;
		}

	g=&games[x-1];
	cprintf("\n\r\n\rGAME #%03d/%03d\n\r",x,gamecount);
	cprintf("SPORT %2.2s    DATE %2u/%2u/%4u  VISIT %3u   HOME %3u \n\r\n\r", g->sport , g->day.da_mon, g->day.da_day, g->day.da_year , g->visitteam , g->hometeam );

	cprintf("TEAM NAME  -   %-12.12s             %-12.12s\n\r", teams[g->visitteam], teams[g->hometeam] );
	cprintf("SCORE      -   %-12.03u             %-12.03u\n\r", g->visitscore  , g->homescore);
	cprintf("PITCHER    -   %-12.12s             %-12.12s\n\r", g->visitpitch  , g->homepitch);
	cprintf("MESSAGE    -   %-12.06s             %-12.06s\n\r", g->visitmessage, g->homemessage);
	cprintf("CPITCHER   -   %-12.12s             %-12.12s\n\r", g->visitcpitch , g->homecpitch);
	cprintf("LINE       -   %-12.04s             %-12.04s\n\r", g->visitline   , g->homeline);

	cprintf("\n\rOVERUNDER    %-4.4s\n\r",g->overunder);

	cprintf("\n\rSTATUS %2d    REMAIN %2d     UPDATE %d \n\r",g->status,g->remain,g->updateflag);
	cprintf("SEQ    %2u    BOX    %2c     TIME   %5.5s \n\r\n\n",g->seq,g->box,g->time);
	}

void readlines(char *filename) {
	FILE *f;
	static char buffer[101];
	f=fopen(filename,"r");
	if (!f) return;
	while( fgets(buffer,100,f) ) {
		struct game g;
		struct game *h;
		int len;
		len=strlen(buffer);
		if (!len) continue;
		buffer[len-1]=0x00;		/* Kill trailing \n */
		str2date( &g.day , buffer );
		strncpy( g.sport , buffer+11 , 2);
		g.visitteam = atoi(buffer+26);
		g.hometeam  = atoi(buffer+30);
		h=bsearch( &g , games , gamecount , sizeof(struct game) , (fptr) gamecmp );
		if (!h) {
			h=binsert( &g , games, &gamecount , sizeof( struct game) , (fptr) gamecmp );
			if (!h) {
				error("Could not insert game in lines");
				continue;
				}
			else {
				blankgame( h );
				h->status=0;		/* New game, not started */
				h->updateflag=0;
				}
			}
		h->box = buffer[14];
		h->seq = atoi( buffer+22 );
		strncpy(h->time , buffer+16 , 5);
		strncpy(h->visitline ,buffer+34,4);
		if (h->visitline[0]==' ') h->visitline[0]=0x00;
		strncpy(h->homeline  ,buffer+39,4);
		if (h->homeline[0]==' ') h->homeline[0]=0x00;
		strncpy(h->overunder ,buffer+44,4);
		if (h->status==0 && len>50) {				/* Don't cover pitcher in active game */
			strncpy(h->visitpitch,buffer+49,10);
			h->visitpitch[10]=0x00;
			strncpy(h->visitmessage,"PROBL",6);
			strncpy(h->homepitch ,buffer+63,10);
			h->homepitch[10]=0x00;
			strncpy(h->homemessage,"PROBL",6);
			}
		memset( buffer , 0 , 100);
		}
	fclose(f);
	}

void trigger(void) {
	struct game *g;
	semaphore=1;
	memcpy( gamebuffer , games , sizeof(struct game) * gamecount );
	g=games+gamecount;
	while (g>games) {
		g--;
		g->updateflag=0;
		}
	semaphore=0;
	triggerprocess();
	}


char *inning( struct game *g ) {
	static char buffer[10];
	if (g->status==99) return("FL");	/* Final */
	if (g->status==98) return("SS");	/* Game suspended */
	if (g->status==97) return("PP");	/* Postponed */
	if (g->status==96) return("CX");	/* Canceled */
	if (g->status==95) return("RD");	/* Rain delay */
	if (g->status==94) return("HT");	/* Half time */
	if ( !strncmp( g->sport , "AA" , 2) ) {
		int inning=((g->status-1) /2)+1;
		if (inning<10) sprintf( buffer , "%s%1u" , g->status & 1 ? "T":"B" , inning );
		else sprintf( buffer , "%s%2u" , g->status & 1 ? "T":"B" , inning );
		return(buffer);
		}
	if ( !strncmp( g->sport, "AF" , 2) || !strncmp( g->sport, "AG" , 2) || !strncmp( g->sport, "AB" , 2)) {
		if (g->status<10) sprintf( buffer , "Q%1u" , g->status );
		else sprintf( buffer , "Q%1u" , g->status );
		return( buffer );
		}
	return("??");
	}

char *( struct game *g ) {
	static char buffer[10];
	if (g->status==99) return("FL");	/* Final */
	if (g->status==98) return("SS");	/* Game suspended */
	if (g->status==97) return("PP");	/* Postponed */
	if (g->status==96) return("CX");	/* Canceled */
	if (g->status==95) return("RD");	/* Rain delay */
	if (g->status==94) return("HT");	/* Half time */
	if ( !strncmp( g->sport , "AA" , 2) ) {
		int inning=((g->status-1) /2)+1;
		if (inning<10) sprintf( buffer , "%s%1u" , g->status & 1 ? "T":"B" , inning );
		else sprintf( buffer , "%s%2u" , g->status & 1 ? "T":"B" , inning );
		return(buffer);
		}
	if ( !strncmp( g->sport, "AF" , 2) || !strncmp( g->sport, "AG" , 2) || !strncmp( g->sport, "AB" , 2)) {
		if (g->status<10) sprintf( buffer , "Q%1u" , g->status );
		else sprintf( buffer , "Q%1u" , g->status );
		return( buffer );
		}
	return("??");
	}

char *remain(struct game *g) {
	static char buffer[20];
	int minutes,seconds;
	if (g->remain==-1) return("");

	minutes = g->remain/60;
	seconds = g->remain - (minutes*60);

	if (minutes >= 10 ) sprintf(buffer,"%3um",g->remain/60);
	else sprintf(buffer,"%u:%02u",minutes,seconds);
	return( buffer );
	}

char beepbuff[6500];			/* Buffer for outgoing message */
char *beepnext=beepbuff;		/* Last used byte in BeepBuffer */
int beeplen=0;				/* Len on BeepBuff */

int bprintf( char *format , ... ) {		/* Print to beeper Buffer */
	va_list ap;
	int ret;
	va_start( ap, format );
	ret=vsprintf( beepnext , format , ap );
	va_end(ap);
	beepnext += ret;
	beeplen  += ret;
	return(ret);
	}

void bzap(void) {		/* ZAP the beepBuffer */
	beepnext=beepbuff;
	beeplen=0;
	*beepnext=0x00;
	}

int refresh=0;		/* Should dump all boxes, or just ones that changed */

int currentbox=0;

void bdump(int changed) {		/* Is there any new data here? */
	char name[12];
	currentbox++;
	if (currentbox>12) return;
	sprintf( name, "BOX%02d.OUT" , currentbox );
	if (changed || refresh) {
		f=fopen(name,"wb");
		if (f) {
			cleanstr( beepbuff );
			fprintf(f,"%s",beepbuff);
			fclose(f);
			}
		else error("Couldn't open BOX file");
		}
	else remove( name );		/* Blank it, in case */
	}

void putgame( struct game *g ) {
	bprintf( "%02u%2.2s-%-8.8s %2u %3.3s " ,  g->seq, inning(g) , teams[g->visitteam] , g->visitscore, (*(g->visitline)) ? g->visitline : g->overunder );
	bprintf( "%-4s %-8.8s %2u %3.3s " ,  remain(g) , teams[g->hometeam] , g->homescore, (*(g->homeline)) ? g->homeline : g->overunder );
	}

void putbbgame( struct game *g) {
	bprintf( "%03u-%-8.8s%3u %4.4s " , g->seq, teams[g->visitteam] , g->visitscore, (*(g->visitline)) ? g->visitline : g->overunder);
	bprintf( "%-3.3s %-8.8s%3u %4.4s " , inning(g) , teams[g->hometeam] , g->homescore, (*(g->homeline)) ? g->homeline : g->overunder);
	bprintf( "%-12.12s  %6.6s "   ,   g->visitpitch , g->visitmessage );
	bprintf( "%-12.12s  %6.6s "   ,   g->homepitch , g->homemessage );
	}

void putbbfgame( struct game *g) {
	bprintf( "%03u-%-8.8s%3u %4.4s " , g->seq , teams[g->visitteam] , g->visitscore, (*(g->visitline)) ? g->visitline : g->overunder);
	bprintf( "FNL %-8.8s%3u %4.4s " ,  teams[g->hometeam] , g->homescore, (*(g->homeline)) ? g->homeline : g->overunder);
	bprintf( "%-12.12s  CONFRM "   ,   g->visitcpitch );
	bprintf( "%-12.12s  CONFRM "   ,   g->homecpitch );
	}

void putline( struct game *g) {
	bprintf( "%03u  -%-8.8s  %4.4s " ,  g->seq , teams[g->visitteam] ,(*(g->visitline)) ? g->visitline : g->overunder);
	bprintf( "%-5.5s %-8.8s  %4.4s " ,  g->time , teams[g->hometeam] , (*(g->homeline)) ? g->homeline : g->overunder);
	}

void putbbline( struct game *g) {
	bprintf( "%03u  -%-8.8s  %4.4s " , g->seq , teams[g->visitteam] ,  (*(g->visitline)) ? g->visitline : g->overunder);
	bprintf( "%5.5s %-8.8s  %4.4s " ,  g->time , teams[g->hometeam] ,  (*(g->homeline)) ? g->homeline : g->overunder);
	bprintf( "%-10.10s  PROBABLE "   ,  g->visitpitch );
	bprintf( "%-10.10s  PROBABLE "   ,  g->homepitch );
	}           /*1234512123456781234 */
		    /*12345678901234567890*/

unsigned linecount;

struct game *lines[200];		/* The lines to go in each mailbox */

int priority(int status) {		/* The order status code go into the lines */
	if (!status) return(4);
	if (status>95) return(5);
	return(3);
	}

int linecmp( const struct game **g1, const struct game **g2) {
	int x;
	x=strncmp( (*g1)->sport , (*g2)->sport , 2 );
	if (x) return(x);
	x=priority( (*g1)->status ) - priority( (*g2)->status );
	if (x) return(x);
	return( (*g1)->seq - (*g2)->seq );
	}

void printline( struct game *g ) {
	if (!strncmp(g->sport,"AA",2)) {
		if (g->status==0) putbbline(g);
		else if (g->status>95) putbbfgame( g );
		else putbbgame( g );
		}
	else {
		if (g->status==0) putline( g);
		else putgame( g );
		}
	}

void sortlines(void) {
	qsort( lines, linecount , sizeof( struct game * ) , (fptr) linecmp );
	}

void printlines(void) {		/* Sorts and Prints out enties in lines */
	struct game **l;
	sortlines();
	l=lines;
	while (linecount--) {
		printline( *(l++) );
		}
	}


char filename(void) {


void ticker(void) {
	struct game *g;
	struct game *buffer,*end;
	static struct time t;
	struct game **l;
	int flag=0;
	int page=0;
	buffer=gamebuffer;
	end=buffer+gamecount;
	l=lines; linecount=0;
	for(g=buffer;g<end;g++) {
		if ( g->updateflag) {
			*(l++)=g;
			linecount++;
			flag=1;
			}
		}
	sortlines();
	gettime( &t );
	bzap();
		 /*1234567890123456 */
	sprintf(t,"TICKER %2u:%02u     ",t.ti_hour,t.ti_min);
		 /*12345678901234567890*/
	strcat(t,"    Recent Changes   ");
	strcat(t,"     (c)1992 SBI     ");
	strcat(t," Info: 212-425-8888  ");

	if (!flag) bprintf(" *NO RECENT UPDATES*");

	else {

	bdump(currentbox++);
	}

void dump(int refresh) {
	int flag=0;
	char title[100];
	trigger();
	gettime( &t );
	bzap();
		 /*1234567890123456 */
	sprintf(t,"TICKER %2u:%02u     ",t.ti_hour,t.ti_min);
		 /*12345678901234567890*/
	strcat(t,"    ------------     ");
	strcat(t,"     (c)1992 SBI     ");
	strcat(t," Info: 212-425-8888  ");
	l=lines; linecount=0;
	for(g=buffer;g<end;g++) {
		if ( g->updateflag) {
			*(l++)=g;
			linecount++;
			flag=1;
			}
		}
		if (!flag) bprintf(" *NO RECENT UPDATES*");
	else printlines();
		bdump(f);
		fclose(f);
		}

	f=fopen("BBALLNF.DAT","wb");
	if (f) {
		bzap();
			 /*1234567890123456 */
		bprintf("NATIONAL LEAGUE  ");
			 /*12345678901234567890*/
		bprintf("    Future Games     ");
		bprintf("     (c)1992 SBI     ");
		bprintf(" Info: 212-425-8888  ");
		l=lines; linecount=0;
		for(g=buffer;g<end;g++) {
			if (!strncmp(g->sport,"AA",2) && g->box=='N' && datecmp(&(g->day),&today)>0 ) {
				*(l++)=g;
				linecount++;
				}
			}
		printlines();
		bdump(f);
		fclose(f);
		}

	f=fopen("BBALLN.DAT","wb");
	if (f) {
		bzap();
			 /*1234567890123456 */
		bprintf("NATIONAL LEAGUE  ");
			 /*12345678901234567890*/
		bprintf("    Today's Games    ");
		bprintf("     (c)1992 SBI     ");
		bprintf(" Info: 212-425-8888  ");
		l=lines; linecount=0;
		for(g=buffer;g<end;g++) {
			if (!strncmp(g->sport,"AA",2) && g->box=='N' && !datecmp(&(g->day),&today) ) {
				*(l++)=g;
				linecount++;
				}
			}
		printlines();
		bdump(f);
		fclose(f);
		}

	f=fopen("BBALLA.DAT","wb");
	if (f) {
		bzap();
			 /*1234567890123456 */
		bprintf("AMERICAN LEAGUE  ");
			 /*12345678901234567890*/
		bprintf("    Today's Games    ");
		bprintf("     (c)1992 SBI     ");
		bprintf(" Info: 212-425-8888  ");
		l=lines; linecount=0;
		for(g=buffer;g<end;g++) {
			if (!strncmp(g->sport,"AA",2) && g->box=='A' && !datecmp(&g->day,&today)) {
				*(l++)=g;
				linecount++;
				}
			}
		printlines();
		bdump(f);
		fclose(f);
		}

	f=fopen("BBALLAF.DAT","wb");
	if (f) {
		bzap();
			 /*1234567890123456 */
		bprintf("AMERICAN LEAGUE  ");
			 /*12345678901234567890*/
		bprintf("    Future Games     ");
		bprintf("     (c)1992 SBI     ");
		bprintf(" Info: 212-425-8888  ");
		l=lines; linecount=0;
		for(g=buffer;g<end;g++) {
			if (!strncmp(g->sport,"AA",2) && g->box=='A' && datecmp(&g->day,&today)>0 ) {
				*(l++)=g;
				linecount++;
				}
			}
		printlines();
		bdump(f);
		fclose(f);
		}

	f=fopen("FOOT.DAT","wb");
	if (f) {
		bzap();
			 /*1234567890123456 */
		bprintf("PRO NFL FOOTBALL ");
			 /*12345678901234567890*/
		bprintf("    Today's Games    ");
		bprintf("     (c)1992 SBI     ");
		bprintf(" Info: 212-425-8888  ");
		l=lines; linecount=0;
		for(g=buffer;g<end;g++) {
			if (!strncmp(g->sport,"AB",2) && !datecmp(&g->day,&today)) {
				*(l++)=g;
				linecount++;
				}
			}
		printlines();
		bdump(f);
		fclose(f);
		}

	f=fopen("FOOTF.DAT","wb");
	if (f) {
		bzap();
			 /*1234567890123456 */
		bprintf("PRO NFL FOOTBALL ");
			 /*12345678901234567890*/
		bprintf("    Future Games     ");
		bprintf("     (c)1992 SBI     ");
		bprintf(" Info: 212-425-8888  ");
		l=lines; linecount=0;
		for(g=buffer;g<end;g++) {
			if (!strncmp(g->sport,"AB",2) && datecmp(&g->day,&today)>0 ) {
				*(l++)=g;
				linecount++;
				}
			}
		printlines();
		bdump(f);
		fclose(f);
		}

	/****** Collage Football
	l=lines; linecount=0;
	for(g=buffer;g<end;g++) {
		if (!strncmp(g->sport,"AB",2)) {
			*(l++)=g;
			linecount++;
			}
		}
	printlines(f);
	fclose(f);
	*/
	}

void debugload(char *filename) {
	FILE *f;
	f=fopen(filename,"rb");
	if (f) gamecount=fread(games,sizeof(struct game),GAMECOUNT,f);
	fclose(f);
	}

void debugdump(char *filename) {
	FILE *f;
	f=fopen(filename,"wb");
	fwrite(games,sizeof(struct game),gamecount,f);
	fclose(f);
	}

int setinterval( int interval ) {
	char buffer[10];
	cprintf("Interval=%d. New Interval? (Press Return to leave unchanged)",interval);
	fgets( buffer , 5 , stdin);
	if (isdigit(*buffer)) interval=atoi(buffer);
	cprintf("\n\rInterval set to %d minutes.\n\r", interval );
	return(interval);
	}

void main(void) {
	int beepnow=0;
	int done=0;
	int game=0;
	int interval=5;
	int c;
	printf("SPORTFEED, (c)1992 Joshua Levine\n\r");
	games=calloc( GAMECOUNT , sizeof( struct game ));
	gamebuffer=calloc( GAMECOUNT , sizeof( struct game ));
	if (!games || !gamebuffer) {
		cprintf("Not enough memory for GAMES.\n\r");
		return;
		}
	getdate( &today);
	debugload("debug.dat");
	printf("Reading team data\n\r");
	readteams("TEAMS.DAT");
	printf("Reading lines\n\r");
	readlines("LINES.TBL");
	initcom(COM2INT,COM2BASE);
	while (!done) {
		struct time t;
		checkcom();
		if ( kbhit()) {
			switch (toupper(getch())) {
				case 0x1b: done=1; break;
				case 'S': system("c:\\command.com"); break;
				case 'B': beepnow=1; break;
				case 'N': if (game<gamecount) {
						game++;
						}
					showgame(game);
					break;
				case 'P': if (game>1) {
						game--;
						}
					showgame(game);
					break;
				case 'V':
					cprintf("\n\rViewing incoming data... Press any key to continue\n\r\n\r");
					dumpmode=1;
					getch();
					dumpmode=0;
					cprintf("\n\r\n\r");
					break;
				case 'E':
					cprintf("\n\r");
					c=nexterror;
					do {
						c++;
						if (c==ERRORCOUNT) c=0;
						cprintf("%d - %s\n\r",c,errors[c]);
						} while (c!=nexterror);
					break;
				case 'I':
					interval=setinterval(interval);
					break;
				}
			}

		gettime(&t);
		cprintf( "ESC-Quit S-Shell B-Beep N-Next P-Prev E-Errors V-View I-Interval Now %2u:%02u \r", t.ti_min , t.ti_sec);
		if ( beepnow || ( (interval && (t.ti_min%interval) == 0) && (t.ti_sec<20) ) ) {
			readlines("LINES.TBL");
			dump();
			system("c:\\command.com /cBEEP.BAT");
			beepnow=0;
			}
		};
	killcom();
	debugdump("debug.dat");
	printf("\n\rGoodbye.\n\r");
	}
